import { toString } from "lodash"

const escape_string = (v: string): string => {
    if (/^ |[,"\r\n]| $/.test(v)) {
        return '"' + v.replaceAll('"', '""') + '"'
    } else {
        return v
    }
}

const encode = (v: any): string => {
    return escape_string(toString(v))
}

export const datasheet2csv = <T>(
    itemList: T[],  // 对象列表
    converter: (item: T) => any[], // 从单个对象提取需要的项目
    headers?: string[],  // 表头
): Blob => {
    /* 
     * 参照 RFC4180: Common Format and MIME Type for Comma-Separated Values (CSV) Files
     * url: https://datatracker.ietf.org/doc/html/rfc4180 
     * 然而 Microsoft Excel 不完全遵守 RFC4180，因此还有一些特殊情况要考虑
     */
    const eol = '\r\n', sep = ',', bom = '\uFEFF'
    const csv = [bom]
    if (headers) {
        csv.push(headers.map(escape_string).join(sep) + eol)
    }
    itemList.forEach((item) => {
        const strings = converter(item)
        csv.push(strings.map(encode).join(sep) + eol)
    })
    const blob = new Blob(csv, { type: 'text/csv;charset=utf-8' })
    return blob
}

export default datasheet2csv